/*
 * @copyright   © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages/productcomparison/ui/comparelink
 */

/*jsling browser: true, nomen: true*/
/*global define, window*/

define("de_epages/productcomparison/ui/comparelink", [
    "jquery",
    "de_epages",
    "util/storage",

    "jquery/ui/widget"
], function ($, de_epages, storage) {

    'use strict';

    var warn = (window.console && window.console.warn) || function () {
            return;
        };

    $.widget('ui.productcomparisonUiComparelink', {

        options: {
            maxItemNo:              8,
            productID:              undefined
        },

        _create: function () {
            var self = this,
                elem = self.element,
                productID = self.options.productID;

            /* check product id */
            if (!self._check(productID, elem)) {
                return;
            }

            /* add 'click' event handler on associated 'Compare' link */
            elem.on('click', function (event) {
                self._handleClick(event, elem);
            });
        },

        _check: function (productID) {
            if (productID === undefined || isNaN(parseInt(productID, 10))) {
                warn('ProductComparison: parameter \'productID\' and/or \'compare\' link missing; product comparison is removed from SF.');

                this.element.remove();

                return false;
            }

            return true;
        },

        _handleClick: function (event, target) {
            var productIDs = storage.sessionStorage('Product::ProductComparison') || [],
                url = target.attr('href');

            event.preventDefault();

            this._addProductToComparisonList(this.options.productID, productIDs);

            storage.sessionStorage('Product::ProductComparison', productIDs);

            $.each(productIDs, function (index, value) {
                url += '&SelectedObjectID=' + value;
            });

            window.location = url;
        },

        _addProductToComparisonList: function (productID, productIDs) {
            var index = $.inArray(productID, productIDs);

            if (index > -1) {  // deletes former occurrence
                productIDs.splice(index, 1);
            }

            productIDs.push(productID);

            while (productIDs.length > this.options.maxItemNo) {
                productIDs.shift();
            }
        },

        destroy: function () {
            this.off('click');

            return this.Inherited(arguments);
        }

    });

    return de_epages;

});
