/**
 * The `ep.uiProgressbarCircle` widget creates an animated progress bar. This display uses a circle
 * for visualisation, which is filled from a freely selectable start value to a freely selectable
 * end value.
 *
 * ### Examples
 * Creates an animated progress bar circle using all available options.
 *
 * JavaScript:
 *
 *     ep('.ep-progressbar-wrapper').uiProgressbarCircle({
 *         start: 0,
 *         stop: 50,
 *         rotate: 0,
 *         value: 5000,
 *         currency: 'EUR',
 *         text: 'finished!',
 *         theme: 'light',
 *         radius: 45,
 *         fontSize: 0.8,
 *         strokeWidth: 7,
 *         ease: 'quad',
 *         speed: 1,
 *         colorCircleFront: '#f00',
 *.        colorCircleBack: '#ccc'
 *     });
 *
 * HTML:
 *
 *     <div class="ep-progressbar-wrapper"></div>
 *
 *
 * @class jQuery.ui.uiProgressbarCircle
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses jquery.ui.widget
 * @uses jquery.tmpl
 * @uses jquery.i18n
 * @since 7.62.0
 */

/**
 * Starts the animation.
 *
 * @method start
 * @member jQuery.ui.uiProgressbarCircle
 *
 * @since 7.62.0
 */

/**
 * Resets the animation.
 *
 * @method reset
 * @member jQuery.ui.uiProgressbarCircle
 *
 * @since 7.62.0
 */

/**
 * @cfg {String} text Text shown after animation ended.
 */

/**
 * @cfg {String} colorCircleFront Color of the SVG circle in the front (default: undefined => CSS is used).
 */

/**
 * @cfg {String} colorCircleBack Color of the SVG circle in the background (default: undefined => CSS is used).
 */

/**
 * @cfg {Number} strokeWidth Width of the circle stroke.
 */

/**
 * @cfg {Number} value Value that is reached and shown at the end of the animation. If *undefined* the current percentage value is shown.
 */

/**
 * @cfg {String} currency Currency string.
 */

/**
 * @cfg {String|null} theme Name of the used theme (default: null, possible: 'light').
 */

/**
 * @cfg {Number} start Start value of the animation (default: 0).
 */

/**
 * @cfg {Number} stop End value of the animation (default: 100)
 */

/**
 * @cfg {Number} radius Radius of the animation bar circle.
 */

/**
 * @cfg {Number} speed Float value to control the animation speed (default: 1, possible: 0 .. 1 => slower, greater than 1 => faster)
 */

/**
 * @cfg {String} fontSize Float value to control the relative font-size (default: 1, possible: 0 .. 1 => smaller, greater than 1 => bigger)
 */

/**
 * @cfg {Number} rotate Angle in degree of the start position of the animation (default: 0)
 */

/**
 * @cfg {String} ease Name of the animation method (possible: 'sine', 'quad', 'linear', default: 'sine')
 */

/**
 * @cfg {Boolean} auto If *auto* is set to *true* the animation starts automatically after widget is initialized (default: false)
 */

/**
 * @cfg {Boolean} debut If *debug* is set to *true* the debug mode is acitve (default: false)
 */

/**
 * See `jQuery.ui.uiProgressbarCircle` for details.
 *
 * @param {String} text Text shown after animation ended.
 * @param {String} colorCircleFront Color of the SVG circle in the front (default: undefined => CSS is used).
 * @param {String} colorCircleBack Color of the SVG circle in the background (default: undefined => CSS is used).
 * @param {Number} strokeWidth Width of the circle stroke.
 * @param {Number} value Value that is reached and shown at the end of the animation. If *undefined* the current percentage value is shown.
 * @param {String} currency Currency string.
 * @param {String|null} theme Name of the used theme (default: null, possible: 'light').
 * @param {Number} start Start value of the animation (default: 0).
 * @param {Number} stop End value of the animation (default: 100)
 * @param {Number} radius Radius of the animation bar circle.
 * @param {Number} speed Float value to control the animation speed (default: 1, possible: 0 .. 1 => slower, greater than 1 => faster)
 * @param {String} fontSize Float value to control the relative font-size (default: 1, possible: 0 .. 1 => smaller, greater than 1 => bigger)
 * @param {Number} rotate Angle in degree of the start position of the animation (default: 0)
 * @param {String} ease Name of the animation method (possible: 'sine', 'quad', 'linear', default: 'sine')
 * @param {Boolean} auto If *auto* is set to *true* the animation starts automatically after widget is initialized (default: false)
 * @param {Boolean} debut If *debug* is set to *true* the debug mode is acitve (default: false)
 *
 * @method uiProgressbarCircle
 * @member jQuery
 *
 * @since 7.62.0
 */

/*
 * @copyright       © Copyright 2006-2021, epages GmbH, All Rights Reserved.
 *
 * @module          ep.ui.progressbar-circle
 *
 * @revision        $Revision: 1.20 $
 */
/*global define, window*/
/*jslint vars:true, plusplus:true, nomen:true*/
define("ep/ui/progressbar-circle", [
    "jquery",
    "ep",
    "$dict!ep/dict",
    "$tmpl!ep/ui/progressbar-circle",

    "jquery/ui/widget",
    "jquery/tmpl",
    "jquery/i18n"
], function ($, ep, epDict, tmplProgressbarCircle) {
    'use strict';

     /*
     * @dictionary      ep.dict
     *
     * @translation     {Default}
     *                  {TLEs}
     *                  {italic}
     */

    var translations = {
        default: epDict.translate('Default'),
        italic: epDict.translate('italic')
    };

    $.widget('ui.uiProgressbarCircle', {
        tmplElements: {},
        offset: 440,
        step: 0,
        time: 0,
        distance: 0,
        duration: 50,
        speed: 30,

        // for debugging
        debug: false, // true - display tracked time for the whole animation, false - no output
        timestamp: 0,

        options: {
            // text,
            // colorCircleBack,
            // colorCircleFront,
            // strokeWidth,
            // value,
            // currency,
            debug: false,
            theme: null,
            start: 0,
            stop: 100,
            radius: 70,
            speed: 1,  // 0 .. 1 => slower, > 1 => faster
            fontSize: 1,
            rotate: 0,
            ease: 'sine',
            auto: false
        },


        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _create: function () {
            var self = this,
                elem = self.element,
                o = self.options;

            self.offset = Math.ceil(2 * o.radius * Math.PI);
            self.step = o.start;
            self.ease = (typeof self['_' + o.ease] === 'function') ? ('_' + o.ease) : '_sine';
            // set upper bound of stop value to 100 (percent)
            o.stop = o.stop > 100 ? 100 : o.stop;

            self._buildUI();

            // start automatically if flag is set
            o.auto && self.start();
        },

        /**
         * Method to build the circle progress bar UI.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _buildUI: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                strokeWidth = 1 / 7 * o.radius,
                // width and height of SVG and its wrapper
                dimension = (2 * o.radius + 10),

                renderedTmpl;

            // render template with needed variables
            renderedTmpl = tmplProgressbarCircle({
                radius: o.radius,
                start: o.start,
                text: o.text !== undefined ? o.text : undefined,
                theme: o.theme,
                dimension: dimension,
                rotate: o.rotate - 90,
                currency: o.currency ? o.currency : undefined
            });

            // get DOM-Elements from template
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));

            // clear wrapper and append template
            elem.empty();
            elem.append(renderedTmpl);

            // background circle
            (o.colorCircleBack !== undefined) && self.tmplElements.circleBack.css('stroke', o.colorCircleBack);
            self.tmplElements.circleBack.css('stroke-width', o.strokeWidth !== undefined ? o.strokeWidth : strokeWidth);

            // front circle (animated circle)
            (o.colorCircleFront !== undefined) && self.tmplElements.circleFront.css('stroke', o.colorCircleFront);
            self.tmplElements.circleFront.css('stroke-width', o.strokeWidth !== undefined ? o.strokeWidth : strokeWidth);

            // set font-sizes in relation to radius
            self.tmplElements.percentNumValue.css({
                'font-size': (4/7 * o.fontSize * o.radius) + "px"
            });
            self.tmplElements.percentUnitValue.css({
                'font-size': (12/35 * o.fontSize * o.radius) + "px"
            });

            // reset progress bar
            self.reset();
        },

        /**
         * Overwrite jQuery widget method to set an option.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _setOption: function(key, value) {
            var self = this,
                o = self.options;

            self._superApply(arguments);

            if (value !== undefined) {
                switch (key) {
                    case 'ease':
                        if (typeof self['_' + value] === 'function') {
                            self.ease = '_' + value;
                            o.ease = value;
                            self._super(key, value);
                        }
                        break;
                    case 'start':
                        if (!isNaN(value)) {
                            value = value < 0 ? 0 : value;
                            self._super(key, value);
                        }
                        break;
                    case 'stop':
                        if (!isNaN(value)) {
                            value = value > 100 ? 100 : value;
                            self._super(key, value);
                        }
                        break;
                    case 'value':
                        if (!isNaN(value)) {
                            self._super(key, value);
                        }
                        break;
                }
            }

        },

        /**
         * Animation method.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _progress: function() {
            var self = this,
                o = self.options;

            // get next step using an *easing* method
            self.step = self[self.ease](self.time, o.start, self.distance, self.duration);
            // increment time
            self.time += 1;

            // display current value
            self.tmplElements.percentNumValue.text(
                $.i18n.formatNumber((o.value !== undefined
                    ? (o.value * self.step / o.stop)
                    : self.step), "n0", {region: $.i18n.settings.region}
                )
            );
            // set dash offset to the current step
            self.tmplElements.circleFront.css({
                'stroke-dashoffset': parseInt(self.offset - (self.offset * self.step) / 100)
            });

            if (self.time <= self.duration) {
                window.setTimeout(function() {
                    self._progress();
                }, self.speed);
            } else {
                self._onProgressEnd();
            }
        },

        /**
         * Method to start the progress bar animation.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        start: function() {
            var self = this,
                o = self.options;

            // time tracking
            self.timestamp = (new Date()).getTime();

            // reset and start progress bar animation
            self.reset();
            if (o.stop !== 0) {
                self._progress();
            } else {
                // display current value
                self.tmplElements.percentNumValue.text($.i18n.formatNumber(o.value, "n0", {region: $.i18n.settings.region}));
            }
        },

        /**
         * Method to reset the circle progress bar.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        reset: function() {
            var self = this,
                o = self.options;

            // reset variables to initial values
            self.step = o.start;
            self.time = 0;
            self.distance = o.stop - o.start;
            self.speed = 30 * 1 / o.speed;

            // background circle
            self.tmplElements.circleBack.css({
                'stroke-dasharray': self.offset,
                'stroke-dashoffset': 0
            });

            // front circle (animated circle)
            self.tmplElements.circleFront.css({
                'stroke-dasharray': self.offset,
                'stroke-dashoffset': parseInt(self.offset - (self.offset * o.start) / 100)
            });

            // hide text
            if (o.text !== undefined){
                self.tmplElements.cardText.css({
                    'visibility': 'hidden'
                });
            }

            // reset start value
            self.tmplElements.percentNumValue.text(o.start);
        },

        /**
         * Method that is called after the progress bar animation ended.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _onProgressEnd: function() {
            var self = this,
                o = self.options;

            // set text
            if (o.text !== undefined){
                self.tmplElements.cardText.css({
                    'visibility': 'visible'
                });
            }

            // execute callback
            if (o.onProgressEnd !== undefined && typeof o.onProgressEnd === "function") {
                o.onProgressEnd();
            }

            o.debug && console.log("Time: ", (new Date()).getTime() - self.timestamp);
        },

        /**
         * Animation method - get linear steps.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _linear: function(time, start, distance, duration) {
            return distance * time / duration + start;
        },

        /**
         * Animation method - get sine steps.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _sine: function(time, start, distance, duration) {
            return -distance/2 * (Math.cos(Math.PI * time / duration) - 1) + start;
        },

        /**
         * Animation method - get quad steps.
         *
         * @member jQuery.ui.uiProgressbarCircle
         *
         * @since 7.62.0
         */
        _quad: function(time, start, distance, duration) {
            return -distance * (time /= duration) * (time - 2) + start;
        },

        destroy: function () {
            var self = this,
                elem = self.element;

            self._superApply(arguments);

            elem.empty();
        }

    });

    return ep;
});
